//--------------------------------------------------------------------------------------
// Copyright 2011,2012,2013 Intel Corporation
// All Rights Reserved
//
// Permission is granted to use, copy, distribute and prepare derivative works of this
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  THIS SOFTWARE IS PROVIDED "AS IS."
// INTEL SPECIFICALLY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, AND ALL LIABILITY,
// INCLUDING CONSEQUENTIAL AND OTHER INDIRECT DAMAGES, FOR THE USE OF THIS SOFTWARE,
// INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PROPRIETARY RIGHTS, AND INCLUDING THE
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
//--------------------------------------------------------------------------------------
#ifndef __CPUT_AVSM_H__
#define __CPUT_AVSM_H__

#include <d3dx9math.h>
#include <stdio.h>
#include "CPUT_DX11.h"
#include "CPUTBufferDX11.h"
#include "CPUTGPUTimerDX11.h"
#include <D3D11.h>
#include <xnamath.h>
#include <time.h>
#include "CPUTSprite.h"

#include "AVSMdefines.h"
#include "AVSMDebugView.h"

#include "xnamath.h" 
#include ".\AVSMTechnique\AVSMTechnique.h"

#include "SSAOTechnique.h"

// define some controls
const CPUTControlID ID_MAIN_PANEL = 10;
const CPUTControlID ID_SECONDARY_PANEL = 20;
const CPUTControlID ID_DEBUG_PANEL = 30;

const CPUTControlID ID_FULLSCREEN_BUTTON = 100;
const CPUTControlID ID_VOLUME_SHADOWING_METHOD = 101;
const CPUTControlID ID_SHADOW_TEXTURE_DIMENSION = 102;
const CPUTControlID ID_AVSM_NODE_COUNT = 103;
const CPUTControlID ID_PARTICLE_SIZE = 104;
const CPUTControlID ID_PARTICLE_OPACITY = 105;
const CPUTControlID ID_PAUSE_PARTICLES = 106;
const CPUTControlID ID_ENABLE_STATS = 107;
const CPUTControlID ID_WIREFRAME_PARTICLES = 108;
const CPUTControlID ID_VERTEX_SHADE_PARTICLES = 109;
const CPUTControlID ID_PARTICLETESSELLATION = 110;
const CPUTControlID ID_DEBUG_DISPLAY = 111;
const CPUTControlID ID_DEBUG_DISPLAY_SLICE_SLIDER = 112;
const CPUTControlID ID_USE_SSAO = 113;

const CPUTControlID ID_LIGHT_ORBIT_LONG_SLIDER = 113;
const CPUTControlID ID_LIGHT_ORBIT_LAT_SLIDER = 114;

const CPUTControlID ID_IGNORE_CONTROL_ID = -1;

const CPUTControlID ID_SMRT = 200;
const CPUTControlID ID_WST = 201;
const CPUTControlID ID_PST = 202;

// AVSM defines
#define AVSM_SHADOW_TEXTURE_DIM 256
#define AVSM_START_NODE_COUNT 8

//  globals
D3DXMATRIXA16 gWorldMatrix;
ParticleSystem *gParticleSystem = NULL;

struct AVSMGuiState
{
    eAVSMMethod     Method;
    unsigned int    ShadowTextureDimensions;
    unsigned int    NumberOfNodes;
    float           ParticleSize;
    float           ParticleOpacity;
    bool            ParticlesPaused;
	bool			EnableStats;
	bool			WireFrame;
	bool			tessellate;
	bool			vertexShaderShadowLookup;
	float			TessellationDensity;
    float           LightLatitude;
    float           LightLongitude;
};

// AVSM sample
//-----------------------------------------------------------------------------
class AVSMSample : public CPUT_DX11
{
private:
    float                  mfElapsedTime;
    CPUTAssetSet          *mpSkyBox;
    CPUTAssetSet          *mpAssetSet;
    CPUTCameraController  *mpCameraController;
    

    CPUTAssetSet          *mpShadowCameraSet;
    CPUTRenderTargetDepth *mpShadowRenderTarget;

	float mScreenWidth;
	float mScreenHeight;

    CPUTText              *mpFPSCounter;

    // AVSM technique
    AVSMTechnique                   *mpAVSMTechnique;
    AVSMTechnique::Options          mAppOptions;    

    AVSMGuiState         mCurrentGUIState;

    // controls
    CPUTButton          *mpFullScreen;
    CPUTDropdown        *mpVolumeShadowingMethod;
    CPUTDropdown        *mpShadowTextureDimension;
    CPUTDropdown        *mpAVSMNodeCount;
    CPUTSlider          *mpParticleSize;
    CPUTSlider          *mpParticleOpacity;
    CPUTCheckbox        *mpPauseParticles;
    CPUTCheckbox        *mpWireFrameParticles;
	CPUTCheckbox        *mpVertexShadeParticles;
	CPUTSlider		    *mpParticleTessellation;
    CPUTCheckbox        *mpDebugViewShow;
    //CPUTSlider          *mpDebugViewSliceDepth;  //todo: debug remove

    CPUTSlider          *mpLightLatitudeSlider;
    CPUTSlider          *mpLightLongitudeSlider;

	// Stats Counters
#ifdef AVSM_METRICS
    CPUTCheckbox        *mpEnableStats;
    CPUTText			*mpSMRT;
	CPUTText			*mpWST;
	CPUTText			*mpPST;
#endif

    // Debug stuff    
    bool                mAVSMExtensionAvailable;
    CPUTText            *mpExtensionWarning;
    AVSMDebugView       *mpAVSMDebugView;

    // GPU timers
    CPUTGPUTimerDX11    mGPUTimerAVSMCreation;
    CPUTGPUTimerDX11    mGPUTimerParticlesRendering;
    CPUTGPUTimerDX11    mGPUTimerSolidGeometryRendering;
    CPUTGPUTimerDX11    mGPUTimerAll;

    // Mouse
    bool                mLButtonDown;
    bool                mRButtonDown;

#ifdef SSAO_ENABLE
    // SSAO stuff
    SSAOTechnique       mSSAO;
#endif
    
public:
    AVSMSample() : 
        mpSkyBox(NULL),
        mpAssetSet(NULL),
        mpCameraController(NULL),
        mpShadowCameraSet(NULL),
        mpAVSMTechnique(NULL),
        mpFullScreen(NULL),
        mpVolumeShadowingMethod(NULL),
        mpShadowTextureDimension(NULL),
        mpAVSMNodeCount(NULL),
        mpParticleSize(NULL),
        mpParticleOpacity(NULL),
        mpPauseParticles(NULL),
        mpParticleTessellation(NULL),
        mpVertexShadeParticles(NULL),
		mpWireFrameParticles(NULL),
#ifdef AVSM_METRICS
        mpEnableStats(NULL),
#endif
        mpDebugViewShow(NULL),
        mpShadowRenderTarget(NULL),
        mAVSMExtensionAvailable(false),
        mpExtensionWarning(NULL),
        mpAVSMDebugView(NULL),
        mLButtonDown(false)
		{
		}
    virtual ~AVSMSample()
    {
        // Note: these two are defined in the base.  We release them because we addref them.
        SAFE_RELEASE(mpCamera);
        SAFE_RELEASE(mpShadowCamera);

        SAFE_RELEASE(mpSkyBox);
        SAFE_RELEASE(mpAssetSet);
        SAFE_DELETE( mpCameraController );
        SAFE_RELEASE(mpShadowCameraSet);
        SAFE_DELETE( mpShadowRenderTarget );
        SAFE_RELEASE( mpAVSMDebugView );
    }
    virtual CPUTEventHandledCode HandleKeyboardEvent(CPUTKey key);
    virtual CPUTEventHandledCode HandleMouseEvent(int x, int y, int wheel, CPUTMouseState state);
    virtual void                 HandleCallbackEvent( CPUTEventID Event, CPUTControlID ControlID, CPUTControl *pControl );

    virtual void Create();
    virtual void Shutdown();
    virtual void Render(double deltaSeconds);
    virtual void Update(double deltaSeconds);
    virtual void ResizeWindow(UINT width, UINT height);
#ifdef AVSM_METRICS
	void DisplayStats(AVSMTechnique::FrameContext & frameContext, const FrameStats &frameStats);
#endif

protected:
    void CreateParticles(ID3D11Device *d3dDevice); // taken from main.cpp
    void GetGUIState(AVSMGuiState* pState);
    void SetAVSMFrameContextData(AVSMTechnique::FrameContext& AVSMFrameContext);

};
#endif // __CPUT_AVSM_H__
