//--------------------------------------------------------------------------------------
// Copyright 2013 Intel Corporation
// All Rights Reserved
//
// Permission is granted to use, copy, distribute and prepare derivative works of this
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  THIS SOFTWARE IS PROVIDED "AS IS."
// INTEL SPECIFICALLY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, AND ALL LIABILITY,
// INCLUDING CONSEQUENTIAL AND OTHER INDIRECT DAMAGES, FOR THE USE OF THIS SOFTWARE,
// INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PROPRIETARY RIGHTS, AND INCLUDING THE
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
//--------------------------------------------------------------------------------------
#ifndef __CPUT_AVSMDEBUGVIEW_H__
#define __CPUT_AVSMDEBUGVIEW_H__

#include <stdio.h>
#include "CPUT_DX11.h"
#include "CPUTBufferDX11.h"
#include "CPUTSprite.h"
#include ".\AVSMTechnique\AVSMTechnique.h"

// starting parameters
static const float c_DebugSpriteWidth   = 1.3f;
static const float c_DebugSpriteHeight  = 1.3f;
static const float c_DebugSpriteX       = -0.95f;
static const float c_DebugSpriteY       = 0.95f - c_DebugSpriteHeight;

// matches the constant buffer with the same name in AVSMDebugView.fx 
struct AVSMDebugViewConsts
{
    float SliceDepth;
    float MarkerX;
    float MarkerY;
    float MarkerZ;
    float4 UVZoomMulAdd;
};

// The AVSM slide/depth debug view
//-----------------------------------------------------------------------------
class AVSMDebugView:public CPUTRefCount
{
public:
    AVSMDebugView();
    ~AVSMDebugView();
    void Create(const CPUTControlID PanelID, const CPUTControlID SliderID, bool Visible);
    void CreateSprite();
    void SetMaterial(CPUTMaterial *pMaterial);
    void SetVisibility(bool visible) { mpDebugViewSliceDepth->SetVisibility(visible); };
    CPUTEventHandledCode HandleMouseEvent(int x, int y, int wheel, CPUTMouseState state, bool mLButtonDown, bool mRButtonDown);
    void ReleaseTexturesAndBuffers();
    void Resize(UINT width, UINT height);
    void Render(ID3D11DeviceContext* pContext, AVSMTechnique* pAVSMTechnique, AVSMTechnique::FrameContext* pAVSMFrameContext, CPUTRenderParametersDX* pRenderParams);
    
private:
    CPUTSlider          *mpDebugViewSliceDepth;
    CPUTSprite          *mpDebugSprite;
    CPUTBufferDX11      *mpAVSMDebugViewConsts;
    CPUTMaterial        *mpAVSMDebugViewMaterial;
    CPUTText            *mpExtensionWarning;
    float4              mAVSMDebugViewUVZoomMulAdd;
    float4              mAVSMDebugViewMarkerPos;
    float3              mMouseDragStartPos; // z == 1 if dragging

    
};

#endif //__CPUT_AVSMDEBUGVIEW_H__