//--------------------------------------------------------------------------------------
// Copyright 2011-2013 Intel Corporation
// All Rights Reserved
//
// Permission is granted to use, copy, distribute and prepare derivative works of this
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  THIS SOFTWARE IS PROVIDED "AS IS."
// INTEL SPECIFICALLY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, AND ALL LIABILITY,
// INCLUDING CONSEQUENTIAL AND OTHER INDIRECT DAMAGES, FOR THE USE OF THIS SOFTWARE,
// INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PROPRIETARY RIGHTS, AND INCLUDING THE
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
//--------------------------------------------------------------------------------------
#ifndef __SHADERCONSTANTS_H__
#define __SHADERCONSTANTS_H__

#include <fstream>
#include <iostream>
#include <string>

#include <Windows.h>
#include <d3d9.h>
#include <d3dx9.h>
#include <d3dx9math.h>

// NOTE: Must match layout of shader constant buffer

__declspec(align(16))
struct UIConstants
{
    unsigned int faceNormals;
    unsigned int enableStats;
    unsigned int volumeShadowMethod;
    unsigned int enableVolumeShadowLookup;
    unsigned int pauseParticleAnimaton;
    float        particleSize;
    unsigned int particleOpacity;
    float        dummy4;
    unsigned int vertexShaderShadowLookup;
    unsigned int tessellate;
    unsigned int wireframe;
    unsigned int lightingOnly;
	float  TessellationDensity;               // Edge, inside, minimum tessellation factor and 1/desired triangle size
};

__declspec(align(16))
struct PerFrameConstants
{
    D3DXMATRIX  mCameraWorldViewProj;
    D3DXMATRIX  mCameraWorldView;
    D3DXMATRIX  mCameraViewProj;
    D3DXMATRIX  mCameraProj;
    D3DXVECTOR4 mCameraPos;
    D3DXMATRIX  mLightWorldViewProj;
    D3DXMATRIX  mAvsmLightWorldViewProj;
    D3DXMATRIX  mCameraViewToLightProj;
    D3DXMATRIX  mCameraViewToLightView;
    D3DXMATRIX  mCameraViewToAvsmLightProj;
    D3DXMATRIX  mCameraViewToAvsmLightView;
    D3DXVECTOR4 mLightDir;
	D3DXVECTOR4 mScreenResolution;
    D3DXVECTOR4 mScreenToViewConsts;

    UIConstants mUI;
};

__declspec(align(16))
struct ParticlePerFrameConstants
{
    float  mScale;                             // Scene scale factor    
    float  mParticleSize;                      // Particles size in (pre)projection space
    float  mParticleOpacity;				   // (Max) Particle contrbution to the CDF
    float  mParticleAlpha;				       // (Max) Particles transparency
    float  mbSoftParticles;				       // Soft Particles Enable-Disable 
    float  mSoftParticlesSaturationDepth;      // Saturation Depth for Soft Particles.      
};

__declspec(align(16))
struct ParticlePerPassConstants
{
    D3DXMATRIX mParticleWorldViewProj;
    D3DXMATRIX mParticleWorldView;
    __declspec(align(16)) D3DXVECTOR3 mEyeRight;
    __declspec(align(16)) D3DXVECTOR3 mEyeUp;   
};

__declspec(align(16))
struct LT_Constants
{
    int   mMaxNodes;    
    float mFirstNodeMapSize;
};

__declspec(align(16))
struct AVSMConstants
{   
    D3DXVECTOR4 mMask0; 
    D3DXVECTOR4 mMask1;
    D3DXVECTOR4 mMask2;
    D3DXVECTOR4 mMask3;
    D3DXVECTOR4 mMask4;
    float       mEmptyNode;
    float       mOpaqueNodeTrans;
    float       mShadowMapSize;
}; 

#endif // __APP_SHADER_CONSTANTS_H__
