//--------------------------------------------------------------------------------------
// Copyright 2011 Intel Corporation
// All Rights Reserved
//
// Permission is granted to use, copy, distribute and prepare derivative works of this
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  THIS SOFTWARE IS PROVIDED "AS IS."
// INTEL SPECIFICALLY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, AND ALL LIABILITY,
// INCLUDING CONSEQUENTIAL AND OTHER INDIRECT DAMAGES, FOR THE USE OF THIS SOFTWARE,
// INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PROPRIETARY RIGHTS, AND INCLUDING THE
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
//--------------------------------------------------------------------------------------
#include "CPUTFrameStats.h"

// Constructor
//------------------------------------------------------------------------------
CPUTFrameStats::CPUTFrameStats():m_Low(0),m_High(-1),m_Head(0)
{
    memset(m_pArray, 0, sizeof(CPUTFrameInfo)*CPUT_FRAME_STAT_WINDOW_SIZE);    
}


// 
//------------------------------------------------------------------------------
void CPUTFrameStats::AddFrame(CPUTFrameInfo& FrameEntry)
{
    m_Head+=1;
    m_Head = m_Head % CPUT_FRAME_STAT_WINDOW_SIZE;    
    m_pArray[m_Head] = FrameEntry;
}

// Function returns the averaged FPS for the desired time window.
// Reports the averages of each of the various timer and number of frames it took
// to get to the target time window
//------------------------------------------------------------------------------
CPUTFrameInfo CPUTFrameStats::GetAveragedFPS(double DesiredTimeWindow, unsigned int& NumberOfFrames)
{
    // walk backwards through data and accumulate until you're over the timeWindow desired
    unsigned int SummingIndex= (m_Head-1) % CPUT_FRAME_STAT_WINDOW_SIZE;
    unsigned int WrapAround = m_Head;    
    double sum=0;
    double sum1=0; 
    double sum2=0;

    NumberOfFrames = 0;
    m_High=0;
    m_Low=999999999999;
    while( (sum<DesiredTimeWindow) && (SummingIndex!=WrapAround))
    {
        sum = sum + m_pArray[SummingIndex].TotalFrameTime;
        sum1 += m_pArray[SummingIndex].TechniqueTime1;
        sum2 += m_pArray[SummingIndex].TechniqueTime2;

        // find max and min values in the window
        m_High = max(m_High, m_pArray[SummingIndex].TotalFrameTime);
        m_Low = min(m_Low, m_pArray[SummingIndex].TotalFrameTime);

        NumberOfFrames++;
        SummingIndex--;
        SummingIndex = SummingIndex % CPUT_FRAME_STAT_WINDOW_SIZE;
    }
    CPUTFrameInfo WindowSummary;
    
    // average of frame time
    WindowSummary.TotalFrameTime = sum;

    WindowSummary.TechniqueTime1 = sum1;
    WindowSummary.TechniqueTime1 = sum2;

    return WindowSummary;
}

// 
//------------------------------------------------------------------------------
double CPUTFrameStats::GetHigh()
{
    return m_High;
}

// 
//------------------------------------------------------------------------------
double CPUTFrameStats::GetLow()
{
    return m_Low;
}