

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
//AVSM CODE////////////////////////////////////////////////////////////////////////////////////////////////////

#include "IntelExtensions.hlsl"
#include "GBuffer.hlsl"
#include "AVSM_Gen.hlsl"
#include "AVSM_Resolve.hlsl"
#include "Rendering.hlsl"
#include "ConstantBuffers.hlsl"
#include "ScreenSpace.hlsl"
#include "AVSM.hlsl"

// Texture2D    _Shadow            : register( t1 );

//AVSM CODE////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////


struct AVSMDebugViewConsts
{
    float SliceDepth;
    float MarkerX;
    float MarkerY;
    float MarkerZ;

    float4 UVZoomMulAdd;
};

cbuffer cbAVSMDebugViewConsts : register(b4)
{
   AVSMDebugViewConsts  g_AVSMDebugViewConsts;
};

// ********************************************************************************************************

struct VS_INPUT
{
    float3 Pos     : POSITION;
    float2 Uv      : TEXCOORD0;
};

struct PS_INPUT
{
    float4 Pos     : SV_POSITION;
    float2 Uv      : TEXCOORD0;
};

// ********************************************************************************************************
Texture2D    TEXTURE0 : register( t0 );
SamplerState SAMPLER0 : register( s0 );

// ********************************************************************************************************
PS_INPUT VSMain( VS_INPUT input )
{
    PS_INPUT output = (PS_INPUT)0;
    output.Pos  = float4( input.Pos, 1.0f);
    output.Uv   = input.Uv;
    return output;
}

// ********************************************************************************************************
float4 PSMain( PS_INPUT input ) : SV_Target
{
    float2 zoomModUV = input.Uv * g_AVSMDebugViewConsts.UVZoomMulAdd.xy + g_AVSMDebugViewConsts.UVZoomMulAdd.zw;

    uint2 dim;
	NONCPUT_gAVSMGenClearMaskSRV.GetDimensions(dim[0], dim[1]);    
    const int2 pixelAddr = (int2)(zoomModUV * (float2)dim);
    float ctrlSurface = AVSMGenLoadControlSurfaceSRV(pixelAddr);

    const int2 markerPixelAddr = (int2)(float2( g_AVSMDebugViewConsts.MarkerX, g_AVSMDebugViewConsts.MarkerY ) * (float2)dim );
    float2 markerUVPos = (markerPixelAddr + float2( 0.5, 0.5 )) / (float2)dim;

    float shadow = VolumeSample( mUI.volumeShadowMethod, zoomModUV, g_AVSMDebugViewConsts.SliceDepth );

    float marker;
    // draw marker reticle
    {
        float2 deltaFromMarker = float2(pixelAddr - markerPixelAddr);
        float distFromMarker = length( deltaFromMarker );
        float2 deltaFromMarkerF = float2(zoomModUV * (float2)dim - markerUVPos * (float2)dim);
        float distFromMarkerF = length( deltaFromMarkerF );
        const float markerRadius = 5.7;
        marker = (distFromMarker < markerRadius) * ( abs( abs( deltaFromMarker.x ) - abs( deltaFromMarker.y ) ) < 0.02 );
        marker += saturate( (0.4 - abs(distFromMarkerF - markerRadius)) * 10.0 );
        marker = saturate( marker * (distFromMarker > (1.0) ) );
    }
                                                
    return float4( lerp( shadow.xxx, float3( 1, 0, 0 ), marker * 0.9 ), 0.95 );          
}
