//--------------------------------------------------------------------------------------
// Copyright 2011,2012,2013 Intel Corporation
// All Rights Reserved
//
// Permission is granted to use, copy, distribute and prepare derivative works of this
// software for any purpose and without fee, provided, that the above copyright notice
// and this statement appear in all copies.  Intel makes no representations about the
// suitability of this software for any purpose.  THIS SOFTWARE IS PROVIDED "AS IS."
// INTEL SPECIFICALLY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, AND ALL LIABILITY,
// INCLUDING CONSEQUENTIAL AND OTHER INDIRECT DAMAGES, FOR THE USE OF THIS SOFTWARE,
// INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PROPRIETARY RIGHTS, AND INCLUDING THE
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Intel does not
// assume any responsibility for any errors which may appear in this software nor any
// responsibility to update it.
//--------------------------------------------------------------------------------------

#include <d3dx9math.h>
#include <stdio.h>
#include "CPUT_DX11.h"
#include "CPUTBufferDX11.h"
#include "CPUTGPUTimerDX11.h"
#include "CPUTRenderTarget.h"
#include "CPUTTextureDX11.h"
#include <D3D11.h>
#include <xnamath.h>
#include <time.h>
#include "CPUTSprite.h"

#include "xnamath.h" 

#ifndef __CPUT_SSAO_H__
#define __CPUT_SSAO_H__

//-----------------------------------------------------------------------------
class SSAOTechnique
{
private:
    // SSAO stuff
    CPUTBufferDX11 *        mpSSAOConstantsBuffer;
    CPUTMaterial *          mpMaterialSSAOGenerate;
    CPUTMaterial *          mpMaterialSSAOBlurHalfResA;
    CPUTMaterial *          mpMaterialSSAOBlurHalfResB;
    CPUTMaterial *          mpMaterialSSAOApplyHalfRes;
    CPUTRenderTargetColor * mpScratchHalfResRenderTarget;
    CPUTRenderTargetColor * mpScratch2HalfResRenderTarget;
    CPUTSprite *            mpFullscreenSprite;

public:
    SSAOTechnique() : 
        mpMaterialSSAOGenerate(NULL),
        mpMaterialSSAOBlurHalfResA(NULL),
        mpMaterialSSAOBlurHalfResB(NULL),
        mpMaterialSSAOApplyHalfRes(NULL),
        mpScratchHalfResRenderTarget(NULL),
        mpScratch2HalfResRenderTarget(NULL),
        mpFullscreenSprite(NULL)
		{
		}
    virtual ~SSAOTechnique()
    {
        SAFE_RELEASE( mpSSAOConstantsBuffer );
        SAFE_RELEASE( mpMaterialSSAOGenerate );
        SAFE_RELEASE( mpMaterialSSAOBlurHalfResA );
        SAFE_RELEASE( mpMaterialSSAOBlurHalfResB );
        SAFE_RELEASE( mpMaterialSSAOApplyHalfRes );
        SAFE_DELETE( mpScratchHalfResRenderTarget );
        SAFE_DELETE( mpScratch2HalfResRenderTarget );
        SAFE_DELETE( mpFullscreenSprite );
    }

    void Create( ID3D11Device* d3dDevice, ID3D11DeviceContext * context );
    void ResizeWindow( UINT width, UINT height );
    void UpdateConsts( ID3D11DeviceContext * dxContext, CPUTCamera * camera );
    void GenerateSSAO( ID3D11DeviceContext * dxContext );
    void FullscreenApplySSAO( ID3D11DeviceContext * dxContext );

};
#endif // __CPUT_SSAO_H__
